 
// Cache DOM elements
const inputElements = document.querySelectorAll('[data-model]');
const boundElements = document.querySelectorAll('[data-bind]');

/* --- Bind element --- */

function binder(data) {
  let scope = {}
  // Add data
  if (data) {
    for (const propName of Object.getOwnPropertyNames(data)) {
      scope[propName] = data[propName];
      setBindValue(propName, data[propName]);
    }
  }
  // Loop through input elements
  for (let el of inputElements) {
    if (el.type === 'text' || el.type === 'hidden') {
      // Get property name from each input with an attribute of 'mm-model'
      let propName = el.getAttribute('data-model');

      // Set property update logic
      setPropUpdateLogic(scope, propName);

      // Update bound scope property on input change
      scope[propName] = el.value;
      el.addEventListener('keyup', e => {
        scope[propName] = el.value;
      });
    }
  }
  return scope;
};

function setPropUpdateLogic(scope, prop) {
  if (!scope.hasOwnProperty(prop)) {
    let value;
    Object.defineProperty(scope, prop, {
      // Automatically update bound dom elements when a scope property is set to a new value
      set: (newValue) => {
        value = newValue;

        // Set input elements to new value
        for (let el of inputElements) {
          if (el.getAttribute('data-model') === prop) {
            if (el.type) {
              el.value = newValue;
            }
          }
        }
        // Set all other bound dom elements to new value
        setBindValue(prop, newValue);
      },
      get: () => {
        return value;
      },
      enumerable: true
    })
  }
}

function setBindValue(prop, newValue) {
  for (let el of boundElements) {
    if (el.getAttribute('data-bind') === prop) {
      if (!el.type) {
        el.innerHTML = newValue;
      }
    }
  }
}

/* --- For each --- */